from enum import Enum


class DeviceCodeEnum(str, Enum):
    DOOR_IN = "door_in"
    DOOR_OUT = "door_out"
    REPAIR_IN = "repair_in"
    REPAIR_OUT = "repair_out"
    DETAIL_IN = "detail_in"
    DETAIL_OUT = "detail_out"



from motor.motor_asyncio import AsyncIOMotorClient
from pymongo.errors import ServerSelectionTimeoutError

from .exception.internal_exception import DatabaseInitializeFailureException, DatabaseConnectFailureException
from bson import Decimal128
from bson.codec_options import CodecOptions, TypeDecoder, TypeRegistry
from decimal import Decimal


class Decimal128Decoder(TypeDecoder):
    """自定義 Decimal128 解碼器，自動轉換為 Python Decimal"""
    bson_type = Decimal128
    python_type = Decimal

    def transform_bson(self, value):
        return value.to_decimal()


class MongoDB:
    def __init__(self, user_name: str, password: str, host: str, port: int, db_name: str, server_selection_timeout: int,
                 connection_timeout: int, auth_source: str, ssl: bool = False, ssl_ca_certs: str = None):
        self.client = None
        self.user_name = user_name
        self.password = password
        self.host = host
        self.port = port
        self.db_name = db_name
        self.connection_timeout = connection_timeout
        self.server_selection_timeout = server_selection_timeout
        self.ssl = ssl
        self.ssl_ca_certs = ssl_ca_certs
        self.auth_source = auth_source

    async def connect(self):
        try:
            db_settings = dict(username=self.user_name, password=self.password, host=self.host, port=self.port,
                               connectTimeoutMS=self.connection_timeout,
                               serverSelectionTimeoutMS=self.server_selection_timeout, retryWrites=False,
                               authSource=self.auth_source)
            if self.ssl:
                db_settings["ssl"] = self.ssl

            if self.ssl_ca_certs and self.ssl_ca_certs != "":
                db_settings["tlsCAFile"] = self.ssl_ca_certs

            self.client = AsyncIOMotorClient(**db_settings)
        except ServerSelectionTimeoutError:
            raise DatabaseInitializeFailureException()

    async def close(self):
        if self.client:
            self.client.close()

    def get_database(self):
        if not self.client:
            raise DatabaseConnectFailureException()

        type_registry = TypeRegistry([Decimal128Decoder()])
        codec_options = CodecOptions(type_registry=type_registry)

        return self.client.get_database(self.db_name, codec_options=codec_options)

    async def get_mongodb_uri(self) -> str:
        if self.user_name and self.password:
            uri = f"mongodb://{self.user_name}:{self.password}@{self.host}:{self.port}/{self.db_name}"
        else:
            uri = f"mongodb://{self.host}:{self.port}/{self.db_name}"

        uri += f"?authSource={self.auth_source}&connectTimeoutMS={self.connection_timeout}&serverSelectionTimeoutMS={self.server_selection_timeout}&retryWrites=false"

        if self.ssl:
            uri += "&ssl=true"
        if self.ssl_ca_certs and self.ssl_ca_certs != "":
            uri += f"&tlsCAFile={self.ssl_ca_certs}"

        return uri



import logging
import logging.handlers
import os
import traceback
from abc import ABCMeta, abstractmethod
from contextlib import asynccontextmanager
from functools import lru_cache
from asgi_correlation_id import CorrelationIdMiddleware, CorrelationIdFilter

import dotenv
import watchtower
from beanie import init_beanie
from fastapi import FastAPI, status, Request, APIRouter
from fastapi.exceptions import RequestValidationError
from fastapi.middleware.cors import CORSMiddleware

from . import database, cache_redis
from .const import LOG_FMT, LOG_FMT_NO_DT, LOG_DT_FMT, DEFAULT_LOGGER_NAME, CORRELATION_ID_HEADER_KEY_NAME
from .exception.base_exception import InternalBaseException
from .exception.internal_exception import BadGatewayException, GatewayTimeoutException
from .ext.amazon import aws
from .http.requests import send_webhook_message
from .http.responses import async_response
from .middleware.log_request import LogRequestMiddleware
from .utils import update_dict_with_cast


class BaseFactory(metaclass=ABCMeta):
    DEFAULT_APP_NAME = ""
    API_VERSION = "v0.0.0"

    @abstractmethod
    def init_modules(self, app):
        """
        Each factory should define what modules it wants.
        """

    @abstractmethod
    async def get_document_model_list(self) -> list:
        """
        Each factory should define what model it wants.
        """

    async def init_redis(self, app, app_config):
        if app_config.REDIS_URL:
            redis_cache = cache_redis.CacheRedis(app_config.REDIS_URL)
            await redis_cache.connect()
            app.state.redis = redis_cache.client
            app.state.logger.info("Initialization redis done")
        else:
            app.state.logger.info("skip initialization redis")

    async def init_state_cache(self, app, app_config):
        """
        Each factory should define what model it wants.
        """
        pass

    async def init_state_scheduler_app(self, app, app_config):
        """
        Each factory should define what model it wants.
        """
        pass

    async def close_scheduler_app(self, app, app_config):
        """
        Each factory should define what model it wants.
        """
        pass

    @abstractmethod
    @lru_cache()
    def get_app_config(self):
        """
        Each factory should define what config it wants.
        """

    def create_app(self, title=None) -> FastAPI:

        @asynccontextmanager
        async def lifespan(app: FastAPI):
            # code to execute when app is loading
            await mongodb.connect()
            document_model_list = await self.get_document_model_list()
            await init_beanie(database=app.state.db.get_database(), document_models=document_model_list)
            app.state.logger.info("Database connected")
            await self.init_state_cache(app, self.get_app_config())
            app.state.logger.info("Initialization state cache done")
            await self.init_state_scheduler_app(app, self.get_app_config())
            app.state.logger.info("Initialization state scheduler app done")
            await self.init_redis(app, self.get_app_config())

            yield
            # code to execute when app is shutting down
            await mongodb.close()
            app.state.logger.info("Database disconnected")
            await self.close_scheduler_app(app, self)
            app.state.logger.info("State scheduler closed")

        if title is None:
            title = self.DEFAULT_APP_NAME

        if self.get_app_config().DEBUG:
            app = FastAPI(openapi_url=self.get_app_config().OPEN_API_URL, title=title,
                          debug=self.get_app_config().DEBUG, version=self.API_VERSION, lifespan=lifespan)
        else:
            app = FastAPI(openapi_url=self.get_app_config().OPEN_API_URL, title=title,
                          debug=self.get_app_config().DEBUG, version=self.API_VERSION, lifespan=lifespan, docs_url=None,
                          redoc_url=None)

        self.__load_local_config()
        app.state.config = self.get_app_config()
        self.__setup_main_logger(app, level=logging.DEBUG)
        app.state.aws_session = aws.init_app(app)
        self.__setup_cloud_log(app)
        self.__load_cloud_config(app)

        # 不重要的middleware請加在這之前
        if self.get_app_config().LOGGER_REQUEST_ENABLE:
            app.add_middleware(LogRequestMiddleware, logger=app.state.logger)


        origins = ["*"]

        app.add_middleware(
            CORSMiddleware,
            allow_origins=origins,
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
            expose_headers=[CORRELATION_ID_HEADER_KEY_NAME]
        )

        app.add_middleware(
            CorrelationIdMiddleware,
            header_name=CORRELATION_ID_HEADER_KEY_NAME,
            update_request_header=True
        )

        mongodb = database.MongoDB(self.get_app_config().DATABASE_USERNAME, self.get_app_config().DATABASE_PASSWORD,
                                   self.get_app_config().DATABASE_HOST, self.get_app_config().DATABASE_PORT,
                                   self.get_app_config().DATABASE_NAME,
                                   self.get_app_config().DATABASE_SERVER_SELECTION_TIMEOUT_MS,
                                   self.get_app_config().DATABASE_CONNECT_TIMEOUT_MS,
                                   self.get_app_config().DATABASE_AUTH_SOURCE,
                                   self.get_app_config().DATABASE_SSL, self.get_app_config().DATABASE_SSL_CA_CERTS)

        app.state.db = mongodb
        app.state.config = self.get_app_config()
        self.__init_modules(app)
        self.__init_builtin_api(app)

        @app.exception_handler(InternalBaseException)
        async def http_exception_handler(request: Request, exc: InternalBaseException):
            detail = exc.detail

            if isinstance(exc, BadGatewayException):
                message = f"【{self.DEFAULT_APP_NAME}】Bad gateway, request:{request.__dict__}, exc:{exc}"
                await send_webhook_message(app, message)
            elif isinstance(exc, GatewayTimeoutException):
                message = f"【{self.DEFAULT_APP_NAME}】Gateway timeout, request:{request.__dict__}, exc:{exc}"
                await send_webhook_message(app, message)

            return await async_response(data=detail.get("data"), code=detail.get("code"), message=detail.get("message"),
                                        status_code=exc.status_code)

        @app.exception_handler(RequestValidationError)
        async def validation_exception_handler(request: Request, exc: RequestValidationError):
            data = {"detail": exc.errors()}
            if exc.body:
                data["body"] = exc.body
            return await async_response(data=data,
                                        code="error_unprocessable_entity", message="Validation failed",
                                        status_code=status.HTTP_422_UNPROCESSABLE_ENTITY)

        @app.exception_handler(Exception)
        async def http_exception_handler(request: Request, exc: Exception):
            app.state.logger.warn(f"Exception, request:{request.__dict__}, exc:{exc}")
            app.state.logger.warn(traceback.format_exc())
            message = f"【{self.DEFAULT_APP_NAME}】Unprocessed Exception, request:{request.__dict__}, exc:{exc}"
            await send_webhook_message(app, message)

            return await async_response(code="error_internal_server", message="Internal server error",
                                        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR)

        return app

    def __load_local_config(self):
        dotenv.load_dotenv(override=True)
        update_dict_with_cast(self.get_app_config(), os.environ)

    def __load_cloud_config(self, app):
        if not app.state.aws_session or not self.get_app_config().AWS_PARAMETER_PATH_PREFIX:
            app.state.logger.warn("No AWS session or Parameter Storage configuration, ignore cloud config")
            return

        cloud_conf = {}

        params = {
            "Path": self.get_app_config().AWS_PARAMETER_PATH_PREFIX,
            "Recursive": True,
            "WithDecryption": True
        }

        # AWS only give us 10 parameters per api call
        ssm_client = app.state.aws_session.client("ssm")
        while True:
            result = ssm_client.get_parameters_by_path(**params)
            cloud_conf.update({para["Name"].split("/")[-1]: para["Value"] for para in result["Parameters"]})
            if not result.get("NextToken"):
                break
            params.update({"NextToken": result["NextToken"]})

        update_dict_with_cast(self.get_app_config(), cloud_conf)

    def __init_modules(self, app):
        self.init_modules(app)

    def __setup_main_logger(self, app, logger_name=DEFAULT_LOGGER_NAME, level=logging.INFO):
        logger = self.__setup_logger(app, logger_name, level)
        app.state.logger = logger

    #
    def __setup_logger(self, app, logger_name, level=logging.INFO):
        logger = logging.getLogger(logger_name)
        logger.setLevel(level)

        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(logging.Formatter(fmt=LOG_FMT))
        stream_handler.addFilter(CorrelationIdFilter())
        logger.addHandler(stream_handler)

        return logger

    def __setup_cloud_log(self, app):
        if app.state.aws_session and self.get_app_config().AWS_LOGGROUP_NAME:
            logs_client = app.state.aws_session.client("logs")
            watchtower_handler = watchtower.CloudWatchLogHandler(
                log_group_name=self.get_app_config().AWS_LOGGROUP_NAME,
                boto3_client=logs_client, create_log_group=False)
            watchtower_handler.setFormatter(logging.Formatter(fmt=LOG_FMT_NO_DT, datefmt=LOG_DT_FMT))
            watchtower_handler.addFilter(CorrelationIdFilter())
            app.state.logger.addHandler(watchtower_handler)

    def __init_builtin_api(self, app):
        router = APIRouter(prefix=f"/system", tags=["system"])

        @router.get('/health')
        async def health():
            return await async_response()

        @router.get('/hello')
        async def hello():
            return await async_response(data={"API version": app.version})

        app.include_router(router)



from enum import Enum


class EventCodeEnum(str, Enum):
    MANUAL_CANCEL_SERVICE_TICKET = "manual_cancel_service_ticket"
    MANUAL_TRACING_START_SERVICE_TICKET = "manual_tracing_start_service_ticket"
    MANUAL_TRACING_STOP_SERVICE_TICKET = "manual_tracing_stop_service_ticket"
    MANUAL_CREATE_SERVICE_TICKET = "manual_create_service_ticket"
    MANUAL_IMPORT_RESERVATION_SMWS = "manual_import_reservation_smws"
    MANUAL_MODIFY_USER_SERVICE_TICKET = "manual_modify_user_service_ticket"
    MANUAL_MODIFY_ESTIMATED_ARRIVAL_TIME_SERVICE_TICKET = "manual_modify_estimated_arrival_time_service_ticket"
    MANUAL_MODIFY_ESTIMATED_DELIVERY_TIME_SERVICE_TICKET = "manual_modify_estimated_delivery_time_service_ticket"
    MANUAL_DELETE_ESTIMATED_DELIVERY_TIME_SERVICE_TICKET = "manual_delete_estimated_delivery_time_service_ticket"
    MANUAL_BOOKING_MESSAGE_SERVICE_TICKET = "manual_booking_message_service_ticket"
    MANUAL_DELIVERY_MESSAGE_SERVICE_TICKET = "manual_delivery_message_service_ticket"

    SYSTEM_TRACING_START_SERVICE_TICKET = "system_tracing_start_service_ticket"
    SYSTEM_TRACING_STOP_SERVICE_TICKET = "system_tracing_stop_service_ticket"

    NOSHOW_SERVICE_TICKET_AUTO_CANCEL = "noshow_service_ticket_auto_cancel"
    IMPORT_RESERVATION_CONFLICT_AUTO_CANCEL = "import_reservation_conflict_auto_cancel"
    BOOKING_REMINDING_SERVICE_TICKET = "booking_reminding_service_ticket"
    TODAY_REPAIR_WORKING_SERVICE_TICKET_AUTO_CLOSED = "today_repair_working_service_ticket_auto_closed"

    CAR_IN_ESTABLISHED = "car_in_established"
    CAR_IN_RECEPTION = "car_in_reception"
    CAR_IN_WORKING = "car_in_working"
    CAR_IN_DISPATCH = "car_in_dispatch"
    CAR_IN_DETAILING = "car_in_detailing"
    CAR_IN_NO_TICKET = "car_in_no_ticket"
    CAR_OUT_FINISHED = "car_out_finished"
    CAR_OUT_GENERAL = "car_out_general"
    CAR_OUT_NO_SERVE = "car_out_no_serve"
    CAR_OUT_TEST = "car_out_test"
    CAR_OUT_ACC = "car_out_acc"

    LPR_CHANGE_STATE = "lpr_change_state"
    LPR_CHANGE_POSITION = "lpr_change_position"

    REPAIR_OVERDUE = "repair_overdue"
    DETAILING_OVERDUE = "detailing_overdue"



from pydantic_settings import BaseSettings


class BaseConfig(BaseSettings):
    DEBUG: bool = False
    RUN_PORT: int = 5000
    TIME_ZONE: str = "Asia/Taipei"

    OPEN_API_URL: str = "/openapi.json"

    LOGGER_REQUEST_ENABLE: bool = True

    # Request
    REQUEST_VERIFY_SSL: bool = False
    REQUEST_PROXY: str = ''
    REQUEST_RETRY_COUNT: int = 0
    REQUEST_RETRY_DELAY_INITIAL_SECONDS: float = 1.0
    REQUEST_RETRY_DELAY_FACTOR: float = 1.5
    REQUEST_RETRY_DELAY_RANDOM_JITTER_MIN: float = 0.0
    REQUEST_RETRY_DELAY_RANDOM_JITTER_MAX: float = 0.3
    REQUEST_CONN_POOL_TIMEOUT: float = 5.0
    REQUEST_CONN_TIMEOUT: float = 5.0
    REQUEST_WRITE_TIMEOUT: float = 5.0
    RESPONSE_READ_TIMEOUT: float = 10.0

    # AWS
    AWS_ACCESS_KEY_ID: str = ""
    AWS_SECRET_KEY: str = ""
    AWS_REGION: str = ""
    AWS_PARAMETER_PATH_PREFIX: str = ""
    AWS_LOGGROUP_NAME: str = ""

    # MongoDB
    DATABASE_HOST: str = ''
    DATABASE_USERNAME: str = ''
    DATABASE_PASSWORD: str = ''
    DATABASE_PORT: int = 27017
    DATABASE_NAME: str = ""
    DATABASE_SSL: bool = False
    DATABASE_SSL_CA_CERTS: str = ""
    DATABASE_SERVER_SELECTION_TIMEOUT_MS: int = 5000
    DATABASE_CONNECT_TIMEOUT_MS: int = 10000
    DATABASE_AUTH_SOURCE: str = 'admin'

    # Micro Service
    AUTH_BASE_URL: str = "http://auth-service-api:5000"
    ORGANIZATION_BASE_URL: str = "http://organization-service-api:5000"
    CUSTOMER_BASE_URL: str = "http://customer-service-api:5000"
    CAR_BASE_URL: str = "http://car-service-api:5000"
    RELATIONSHIP_MANAGEMENT_BASE_URL: str = "http://relationship-management-service-api:5000"
    TICKET_BASE_URL: str = "http://ticket-service-api:5000"
    NOTIFY_BASE_URL: str = "http://notify-service-api:5000"
    THIRD_PARTY_BASE_URL: str = "http://third-party-service-api:5000"
    SCHEDULER_BASE_URL: str = "http://scheduler-service-api:5000"
    AUTO_DATA_PROCESSING_BASE_URL: str = "http://auto-data-processing-service-api:5000"

    # Exception Notify
    WEBHOOK_BASE_URL: str = ""
    WEBHOOK_RETRY_COUNT: int = 5
    WEBHOOK_RETRY_DELAY_INITIAL_SECONDS: float = 1.0
    WEBHOOK_RETRY_DELAY_FACTOR: float = 2.0
    WEBHOOK_RETRY_DELAY_RANDOM_JITTER_MIN: float = 0.0
    WEBHOOK_RETRY_DELAY_RANDOM_JITTER_MAX: float = 0.5

    # Default System Account Password
    SYSTEM_ACCOUNT: str = "cruisys"
    SYSTEM_PASSWORD: str = "cs50951855"
    SYSTEM_BRAND: str = "cruisys"
    SYSTEM_ORGANIZATION_ID: str = "cruisys"
    SYSTEM_NAME: str = "系統"
    SYSTEM_TITLE: str = ""

    # Redis URL
    REDIS_URL: str = ""

    class Config:
        case_sensitive = False



import json
import arrow

from datetime import datetime, date

import httpx

from fastapi import status
from fastapi.responses import JSONResponse
from fastapi.encoders import jsonable_encoder
from beanie import Document, Link
from ..const import ARR_HUMAN_READ_FMT, ARR_DATE_FMT


async def async_response(data=None, message=None, code=None, page_no=None, total_num=None, page_size=None,
                         time_zone="UTC", time_format=ARR_HUMAN_READ_FMT, date_format=ARR_DATE_FMT,
                         status_code=status.HTTP_200_OK):
    def _serialize(data):
        if issubclass(type(data), Document):
            link_field_list = []
            datetime_field_list = []
            date_field_list = []
            for field_name in data.__annotations__:
                field_type = getattr(data, field_name)
                if isinstance(field_type, Link):
                    link_field_list.append(field_name)

                if field_name.endswith('_date') or field_name == "birthday":
                    date_field_list.append(field_name)
                elif isinstance(field_type, datetime):
                    datetime_field_list.append(field_name)
                elif isinstance(field_type, date):
                    date_field_list.append(field_name)

            data = json.loads(data.model_dump_json(exclude={"password", "metadata", "otp_code_universal"}, by_alias=False))
            if link_field_list:
                for field_name in link_field_list:
                    if isinstance(data[field_name], dict) and "id" in data[field_name].keys():
                        data[field_name] = data[field_name]["id"]
            for field_name in datetime_field_list:
                if data[field_name]:
                    data[field_name] = arrow.get(data[field_name]).to(time_zone).format(time_format)
            for field_name in date_field_list:
                if data[field_name]:
                    data[field_name] = arrow.get(data[field_name]).to(time_zone).format(date_format)

            if "create_time" in data.keys() and data.get("create_time"):
                data["create_time"] = arrow.get(data["create_time"]).to(time_zone).format(ARR_HUMAN_READ_FMT)

            if "update_time" in data.keys() and data.get("update_time"):
                data["update_time"] = arrow.get(data["update_time"]).to(time_zone).format(ARR_HUMAN_READ_FMT)

        return data

    if isinstance(data, httpx.Response):
        return JSONResponse(status_code=data.status_code, content=data.json())

    ret = {}
    if isinstance(data, list):
        data = [_serialize(d) for d in data]
    else:
        data = _serialize(data)

    data = jsonable_encoder(data)

    ret['code'] = code or "ok"

    ret['message'] = message or "success"

    if page_no and page_size:
        ret['data'] = {
            'page_no': page_no,
            'page_size': page_size,
            'total_num': total_num,
            'page_data': data
        }
    else:
        ret['data'] = data

    return JSONResponse(status_code=status_code, content=ret)



from fastapi import status
from fastapi import HTTPException


class InternalBaseException(HTTPException):
    def __init__(self, status_code: int = status.HTTP_500_INTERNAL_SERVER_ERROR, code: str = "internal_server_error",
                 message: str = "Internal server error", **kwargs):
        detail = {
            "code": code,
            "message": message,
            "data": kwargs,
        }
        super().__init__(status_code=status_code, detail=detail)



from datetime import datetime

from typing import List, Tuple, Union, Dict, Type, Optional, Any, Literal, Union, Set, Mapping
from typing_extensions import Self, TypeAlias, Unpack

import arrow
import pymongo
from beanie import Document
from fastapi import FastAPI
from pydantic import Field, BaseModel

from .operate import Operate
from ..const import DEF_PAGE_SIZE, DEF_PAGE_NO
from ..exception.internal_exception import NoChangeException
from ..common_enum.order_type import OrderTypeEnum
from ..utils import get_current_utc

IncEx: TypeAlias = Union[Set[int], Set[str], Mapping[int, Union['IncEx', bool]], Mapping[str, Union['IncEx', bool]]]


class InternalBaseDocument(Document):
    create_time: datetime = Field(default_factory=get_current_utc)
    update_time: Optional[datetime] = None

    @classmethod
    async def get_pagination_list(cls, app: FastAPI, query: list = None, sort: List[Tuple] = None,
                                  page_size: int = DEF_PAGE_SIZE, page_no: int = DEF_PAGE_NO,
                                  ignore_cache: bool = False, fetch_links: bool = False,
                                  exclude_field_list: List[str] = None):
        if not query:
            final_query = []
        else:
            final_query = query

        if not sort:
            final_sort = [(cls.id, pymongo.ASCENDING)]
        else:
            final_sort = []
            for temp_sort in sort:
                if temp_sort[1] == OrderTypeEnum.ASC:
                    final_sort.append((temp_sort[0], pymongo.ASCENDING))
                elif temp_sort[1] == OrderTypeEnum.DESC:
                    final_sort.append((temp_sort[0], pymongo.DESCENDING))
                else:
                    print(f"order type value error: temp_sort:{temp_sort}")
                    continue
            final_sort.append((cls.id, pymongo.ASCENDING))

        if exclude_field_list:
            # 當需要排除欄位時，使用 Motor 直接操作
            collection = cls.get_motor_collection()
            projection = {field: 0 for field in exclude_field_list}

            # 建立查詢條件
            mongo_query = {}
            for q in final_query:
                if hasattr(q, 'query'):
                    mongo_query.update(q.query)

            # 計算總數
            total_num = await collection.count_documents(mongo_query)
            total_pages = (total_num + page_size - 1) // page_size

            if total_pages == 0:
                page_no = 1
                page_data = []
            else:
                page_no = max(1, min(page_no, total_pages))

                # 執行分頁查詢
                cursor = collection.find(mongo_query, projection).sort(final_sort).skip(
                    (page_no - 1) * page_size).limit(page_size)
                documents = await cursor.to_list(None)

                # 轉換為 Pydantic 模型
                page_data = []
                for doc in documents:
                    try:
                        page_data.append(cls.model_validate(doc))
                    except Exception as e:
                        print(f"模型驗證失敗: {e}")
                        continue
        else:
            # 沒有排除欄位時使用 Beanie 的方法
            total_num = await cls.find(*final_query, ignore_cache=ignore_cache, fetch_links=fetch_links).sort(
                *final_sort).count()

            total_pages = (total_num + page_size - 1) // page_size

            if total_pages == 0:
                page_no = 1
                page_data = []
            else:
                page_no = max(1, min(page_no, total_pages))

                page_data = await cls.find(*final_query, ignore_cache=ignore_cache, fetch_links=fetch_links).sort(
                    *final_sort).limit(page_size).skip((page_no - 1) * page_size).to_list()

        return page_no, page_size, total_num, page_data

    async def update_wrap(self, schema: Union[Dict, Type[BaseModel]]) -> Tuple[
        Operate, 'InternalBaseDocument', 'InternalBaseDocument']:
        if not issubclass(type(schema), dict) and not issubclass(type(schema), BaseModel):
            raise TypeError("Schema must be a subclass of BaseModel or dict")

        original_model = self.model_copy(deep=True)
        delta_dict = schema
        if issubclass(type(schema), BaseModel):
            delta_dict = schema.model_dump(exclude_unset=True, mode="json")

        for key, value in delta_dict.items():
            if hasattr(self, key):
                setattr(self, key, value)

        operate = await Operate.generate_operate(original_model.model_dump(mode="json"), self.model_dump(mode="json"))
        if not operate.add and not operate.remove and not operate.change:
            raise NoChangeException()

        self.update_time = arrow.utcnow().datetime

        await self.save()
        return operate, original_model, self

    @classmethod
    async def get_list(cls, app: FastAPI, query: list = None, sort: List[Tuple] = None, ignore_cache: bool = False,
                       fetch_links: bool = False, exclude_field_list: List[str] = None):
        if not query:
            final_query = []
        else:
            final_query = query

        if not sort:
            final_sort = [(cls.id, pymongo.ASCENDING)]
        else:
            final_sort = []
            for temp_sort in sort:
                if temp_sort[1] == OrderTypeEnum.ASC:
                    final_sort.append((temp_sort[0], pymongo.ASCENDING))
                elif temp_sort[1] == OrderTypeEnum.DESC:
                    final_sort.append((temp_sort[0], pymongo.DESCENDING))
                else:
                    print(f"order type value error: temp_sort:{temp_sort}")
                    continue
            final_sort.append((cls.id, pymongo.ASCENDING))

        if exclude_field_list:
            # 當需要排除欄位時，使用 Motor 直接操作
            collection = cls.get_motor_collection()
            projection = {field: 0 for field in exclude_field_list}

            # 建立查詢條件
            mongo_query = {}
            for q in final_query:
                if hasattr(q, 'query'):
                    mongo_query.update(q.query)

            # 執行查詢
            cursor = collection.find(mongo_query, projection).sort(final_sort)
            documents = await cursor.to_list(None)

            # 轉換為 Pydantic 模型
            data = []
            for doc in documents:
                try:
                    data.append(cls.model_validate(doc))
                except Exception as e:
                    print(f"模型驗證失敗: {e}")
                    continue
        else:
            # 沒有排除欄位時使用 Beanie 的方法
            data = await cls.find(*final_query, ignore_cache=ignore_cache, fetch_links=fetch_links).sort(
                *final_sort).to_list()

        return data

    def model_dump(
            self,
            *,
            mode: Literal['json', 'python'] | str = 'json',
            include: IncEx | None = None,
            exclude: IncEx | None = None,
            context: Any | None = None,
            by_alias: bool = False,
            exclude_unset: bool = False,
            exclude_defaults: bool = False,
            exclude_none: bool = False,
            round_trip: bool = False,
            warnings: bool | Literal['none', 'warn', 'error'] = True,
            serialize_as_any: bool = False,
    ) -> dict[str, Any]:
        return super().model_dump(mode=mode, include=include, exclude=exclude, context=context, by_alias=by_alias,
                                  exclude_unset=exclude_unset, exclude_defaults=exclude_defaults,
                                  exclude_none=exclude_none, round_trip=round_trip, warnings=warnings,
                                  serialize_as_any=serialize_as_any)



import re

from .exception.app_exception import PlateNoFormatException, VinLengthOrFormatException, PhoneFormatException, \
    DateFormatException, EMailFormatException
from .utils import sanitize_plate_no


def verify_and_sanitize_plate_no(value: str, is_require: bool = False):
    if is_require and not value:
        raise PlateNoFormatException()

    if value:
        if not re.match(r'^[A-Za-z0-9]{6,7}$', value):
            raise PlateNoFormatException()
    return sanitize_plate_no(value)


def verify_vin(value: str, is_require: bool = False):
    if is_require and not value:
        raise VinLengthOrFormatException()

    if value:
        if not re.match(r'^(?:[A-Za-z0-9]{7}|[A-Za-z0-9]{10}|[A-Za-z0-9]{17})$', value):
            raise VinLengthOrFormatException()


def verify_phone(value: str, is_require: bool = False):
    if is_require and not value:
        raise PhoneFormatException()

    if value:
        if not re.match(r'^09\d{8}$', value):
            raise PhoneFormatException()


def verify_date(value: str, is_require: bool = False):
    if is_require and not value:
        raise DateFormatException()

    if value:
        if not re.match(r'^\d{4}-\d{2}-\d{2}$', value):
            raise DateFormatException()


def verify_mail(value: str, is_require: bool = False):
    if is_require and not value:
        raise EMailFormatException()

    if value:
        if not re.match(r'^[a-zA-Z0-9._-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$', value):
            raise EMailFormatException()



from enum import Enum


class WebsocketChannelEnum(str, Enum):
    OVER_ALL = "over_all"  # 儀表板
    RECEPTION_CENTER = "reception_center"  # 接待中心
    CAR_MOVEMENT = "car_movement"  # 車輛動態
    USER_BELL_NOTIFICATION = "user_bell_notification"  # 個人小鈴噹



from fastapi import status
from .base_exception import InternalBaseException


class BadGatewayException(InternalBaseException):
    code = "error_bad_gateway"
    message = "Bad gateway"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_502_BAD_GATEWAY, self.code, _message, **kwargs)


class GatewayTimeoutException(InternalBaseException):
    code = "error_gateway_timeout"
    message = "Gateway timeout"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_504_GATEWAY_TIMEOUT, self.code, _message, **kwargs)


class DatabaseInitializeFailureException(InternalBaseException):
    code = "error_database_initialize"
    message = "Database initialize failure"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_500_INTERNAL_SERVER_ERROR, self.code, _message, **kwargs)


class DatabaseConnectFailureException(InternalBaseException):
    code = "error_database_connect"
    message = "Database connect failure"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_500_INTERNAL_SERVER_ERROR, self.code, _message, **kwargs)


class NoChangeException(InternalBaseException):
    code = "error_no_change"
    message = "Document no change"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_200_OK, self.code, _message, **kwargs)


class PermissionDeniedException(InternalBaseException):
    code = "error_permission_denied"
    message = "Permission denied"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_403_FORBIDDEN, self.code, _message, **kwargs)


class ParamValidationException(InternalBaseException):
    code = "error_validation"
    message = "Invalid parameter or incorrect format"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class RedisInitializeFailureException(InternalBaseException):
    code = "error_redis_initialize"
    message = "Redis initialize failure"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_500_INTERNAL_SERVER_ERROR, self.code, _message, **kwargs)


class RedisConnectFailureException(InternalBaseException):
    code = "error_redis_connect"
    message = "Redis connect failure"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_500_INTERNAL_SERVER_ERROR, self.code, _message, **kwargs)



import redis.asyncio as redis

from motor.motor_asyncio import AsyncIOMotorClient
from pymongo.errors import ServerSelectionTimeoutError

from .exception.internal_exception import RedisInitializeFailureException, RedisConnectFailureException


class CacheRedis:
    def __init__(self, redis_url: str):
        self.client = None
        self.redis_url = redis_url

    async def connect(self):
        try:
            connection_pool = redis.ConnectionPool.from_url(self.redis_url, decode_responses=True)
            self.client = redis.Redis(connection_pool=connection_pool)
            await self.client.ping()
        except Exception as e:
            raise RedisInitializeFailureException()



from fastapi import status
from .base_exception import InternalBaseException


class AccountLengthOrFormatException(InternalBaseException):
    code = "error_account_length_or_format"
    message = "Account only allow alphanumeric 30 characters"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class AccountNullPointException(InternalBaseException):
    code = "error_account_not_be_null"
    message = "Account can not be null"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class PasswordNullPointException(InternalBaseException):
    code = "error_password_not_be_null"
    message = "Password can not be null"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class BrandNullPointException(InternalBaseException):
    code = "error_brand_not_be_null"
    message = "Brand can not be null"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class PlateNoFormatException(InternalBaseException):
    code = "error_plate_no_format"
    message = "Plate numbers only allow alphanumeric 6 or 7 characters"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class VinLengthOrFormatException(InternalBaseException):
    code = "error_vin_length_or_format"
    message = "VIN only allow alphanumeric 7 or 10 or 17 characters"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class PhoneFormatException(InternalBaseException):
    code = "error_phone_format"
    message = "Phone only allow start with 09 and add 8 digits"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class DateFormatException(InternalBaseException):
    code = "error_date_format"
    message = "Date format allow YYYY-MM-DD"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class EMailFormatException(InternalBaseException):
    code = "error_email_format"
    message = "Email format error"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class LineUidNullPointException(InternalBaseException):
    code = "error_line_uid_not_be_null"
    message = "Line uid can not be null"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)


class OrganizationIdNullPointException(InternalBaseException):
    code = "error_organization_id_not_be_null"
    message = "Organization id can not be null"

    def __init__(self, message: str = None, **kwargs):
        _message = message or self.message
        super().__init__(status.HTTP_422_UNPROCESSABLE_ENTITY, self.code, _message, **kwargs)



# -*- coding: utf-8 -*-
import json
from datetime import datetime, timezone

import arrow

from .base_config import BaseConfig
from .const import STR_EMPTY, ARR_EXPORT_DATETIME_FMT, STR_DASH, REDIS_LPR_DATA_LIST_PREFIX


def is_today(time, system_time_zone):
    if not time:
        return False

    return arrow.now(system_time_zone).floor("day") == arrow.get(time).to(system_time_zone).floor("day")


def get_tz_day_boundary(date_time, time_zone, out_tz="UTC"):
    """
    傳入date_time，取其在dt_tz時區的當天的floor與ceil，以out_tz時區回傳
    比對區間需使用gte/lte
    """
    date_time = arrow.get(date_time) if date_time else arrow.get()
    tz_time = date_time.to(time_zone)
    return tz_time.floor("day").to(out_tz), tz_time.ceil("day").to(out_tz)


def timestamp_interval(start, end, interval_sec):
    while start < end:
        yield start
        start += interval_sec


def export_time_format(date, time_zone, fmt=ARR_EXPORT_DATETIME_FMT):
    if not date:
        return STR_EMPTY

    return arrow.get(date).to(time_zone).format(fmt)


def update_dict_with_cast(curr_settings: BaseConfig, new_conf: dict):
    if issubclass(type(curr_settings), BaseConfig):
        for key, value in new_conf.items():
            if hasattr(curr_settings, key):
                key_type = type(getattr(curr_settings, key))
                cast_func = key_type if key_type in (str, int) else json.loads
                setattr(curr_settings, key, cast_func(value))


def sanitize_plate_no(plate_no):
    return plate_no.replace(STR_DASH, STR_EMPTY).upper()


def get_current_utc() -> datetime:
    return datetime.now(timezone.utc)


def get_lpr_data_list_redis_key(organization_id, plate_no):
    return f"{REDIS_LPR_DATA_LIST_PREFIX}_{organization_id}_{plate_no}"


async def mask_name(input_string: str, start_index: int = 1, end_index: int = None, mask_char: str = "o"):
    """
    将姓名的前 start_index 个字符保留，其余字符替换为 'o'，用于隐码处理。

    :param mask_char: 隱碼替換字符
    :param input_string: 要隐码的姓名字符串
    :param start_index: 从第几位开始进行隐码处理（默认值为 1）
    :param end_index:  从第几位結束进行隐码处理（默认值为 None）
    :return: 隐码处理后的字符串
    """
    if not input_string:
        return None

    if end_index is None or end_index > len(input_string):
        end_index = len(input_string)

    # 保留前 start_index 个字符
    visible_part_start = input_string[:start_index]

    # 保留从 end_index 之后的字符
    visible_part_end = input_string[end_index:]

    # 创建隐码部分，其余字符替换为 mask_char
    masked_part = mask_char * (end_index - start_index)

    return visible_part_start + masked_part + visible_part_end


def get_dealer_by_organization_id(organization_id: str) -> str:
    """
    將組織代碼為 3 段（例如 'BMW_LONGDER_NH'）時，裁切為前兩段（'BMW_LONGDER'）。
    否則回傳原本的 organization_id。
    """
    if not organization_id or not isinstance(organization_id, str):
        return organization_id

    parts = organization_id.strip().split("_")

    if len(parts) < 2:
        raise ValueError("organization_id is invalid")

    if len(parts) >= 3:
        return "_".join(parts[:2])

    return organization_id

def extract_title(name):
    if '小姐' in name:
        return '小姐'
    elif '女士' in name:
        return '女士'
    elif '先生' in name:
        return '先生'
    else:
        return None

def extract_name(name):
    """从姓名中提取真实姓名"""
    # 检查是否包含 '小姐' 或 '先生' 并提取姓名
    if '小姐' in name:
        return name.split('小姐')[0].strip()
    elif '女士' in name:
        return name.split('女士')[0].strip()
    elif '先生' in name:
        return name.split('先生')[0].strip()
    else:
        return name.strip()